import { IdlAccounts, Program } from "@coral-xyz/anchor";
import { Connection, Keypair, PublicKey, TransactionInstruction } from "@solana/web3.js";
import { AccountAddress, Chain, ChainAddress, ChainsConfig, Contracts, Network, TokenAddress, UnsignedTransaction } from "@wormhole-foundation/sdk-connect";
import { Ntt, NttTransceiver, WormholeNttTransceiver } from "@wormhole-foundation/sdk-definitions-ntt";
import { SolanaChains, SolanaPlatformType, SolanaTransaction, SolanaUnsignedTransaction } from "@wormhole-foundation/sdk-solana";
import { SolanaWormholeCore } from "@wormhole-foundation/sdk-solana-core";
import type { NativeTokenTransfer } from "./anchor-idl/index.js";
import { TransferArgs, nttAddresses } from "./utils.js";
export type Config = IdlAccounts<NativeTokenTransfer>["config"];
export type InboxItem = IdlAccounts<NativeTokenTransfer>["inboxItem"];
export declare class SolanaNttWormholeTransceiver<N extends Network, C extends SolanaChains> implements NttTransceiver<N, C, WormholeNttTransceiver.VAA> {
    readonly manager: SolanaNtt<N, C>;
    readonly address: string;
    constructor(manager: SolanaNtt<N, C>, address: string);
    receive(attestation: WormholeNttTransceiver.VAA, sender?: AccountAddress<C> | undefined): AsyncGenerator<UnsignedTransaction<N, C>, any, unknown>;
    setPeer(peer: ChainAddress<C>): AsyncGenerator<UnsignedTransaction<N, C>, any, unknown>;
}
export declare class SolanaNtt<N extends Network, C extends SolanaChains> implements Ntt<N, C> {
    readonly network: N;
    readonly chain: C;
    readonly connection: Connection;
    readonly contracts: Contracts & {
        ntt?: Ntt.Contracts;
    };
    core: SolanaWormholeCore<N, C>;
    xcvrs: SolanaNttWormholeTransceiver<N, C>[];
    program: Program<NativeTokenTransfer>;
    pdas: ReturnType<typeof nttAddresses>;
    config?: Config;
    constructor(network: N, chain: C, connection: Connection, contracts: Contracts & {
        ntt?: Ntt.Contracts;
    });
    static fromRpc<N extends Network>(provider: Connection, config: ChainsConfig<N, SolanaPlatformType>): Promise<SolanaNtt<N, SolanaChains>>;
    getConfig(): Promise<Config>;
    getCustodyAddress(): Promise<string>;
    initialize(args: {
        payer: Keypair;
        owner: Keypair;
        chain: Chain;
        mint: PublicKey;
        outboundLimit: bigint;
        mode: "burning" | "locking";
    }): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    registerTransceiver(args: {
        payer: Keypair;
        owner: Keypair;
        transceiver: PublicKey;
    }): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    setWormholeTransceiverPeer(peer: ChainAddress, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    setPeer(peer: ChainAddress, tokenDecimals: number, inboundLimit: bigint, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    transfer(sender: AccountAddress<C>, amount: bigint, destination: ChainAddress, queue: boolean, outboxItem?: Keypair): AsyncGenerator<UnsignedTransaction<N, C>, any, unknown>;
    redeem(attestations: Ntt.Attestation[], payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    getCurrentOutboundCapacity(): Promise<bigint>;
    getCurrentInboundCapacity(fromChain: Chain): Promise<bigint>;
    completeInboundQueuedTransfer(fromChain: Chain, transceiverMessage: Ntt.Message, token: TokenAddress<C>, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    getInboundQueuedTransfer(chain: Chain, nttMessage: Ntt.Message): Promise<Ntt.InboundQueuedTransfer<C> | null>;
    createTransferLockInstruction(args: {
        transferArgs: TransferArgs;
        payer: PublicKey;
        from: PublicKey;
        fromAuthority: PublicKey;
        outboxItem: PublicKey;
        config?: Config;
    }): Promise<TransactionInstruction>;
    createTransferBurnInstruction(args: {
        transferArgs: TransferArgs;
        payer: PublicKey;
        from: PublicKey;
        fromAuthority: PublicKey;
        outboxItem: PublicKey;
        config?: Config;
    }): Promise<TransactionInstruction>;
    createReleaseOutboundInstruction(args: {
        payer: PublicKey;
        outboxItem: PublicKey;
        revertOnDelay: boolean;
    }): Promise<TransactionInstruction>;
    createReceiveWormholeMessageInstruction(payer: PublicKey, wormholeNTT: WormholeNttTransceiver.VAA): Promise<TransactionInstruction>;
    createRedeemInstruction(payer: PublicKey, wormholeNTT: WormholeNttTransceiver.VAA): Promise<TransactionInstruction>;
    createReleaseInboundMintInstruction(args: {
        payer: PublicKey;
        chain: Chain;
        nttMessage: Ntt.Message;
        revertOnDelay: boolean;
        recipient?: PublicKey;
    }): Promise<TransactionInstruction>;
    createReleaseInboundUnlockInstruction(args: {
        payer: PublicKey;
        chain: Chain;
        nttMessage: Ntt.Message;
        revertOnDelay: boolean;
        recipient?: PublicKey;
    }): Promise<TransactionInstruction>;
    custodyAccountAddress(mint: PublicKey): Promise<PublicKey>;
    createUnsignedTx(txReq: SolanaTransaction, description: string, parallelizable?: boolean): SolanaUnsignedTransaction<N, C>;
}
//# sourceMappingURL=ntt.d.ts.map