"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nttAddresses = exports.derivePda = exports.U64 = exports.programDataLayout = exports.pubKeyConversion = exports.programDataAddress = exports.BPF_LOADER_UPGRADEABLE_PROGRAM_ID = void 0;
const web3_js_1 = require("@solana/web3.js");
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
const sdk_definitions_ntt_1 = require("@wormhole-foundation/sdk-definitions-ntt");
const bn_js_1 = __importDefault(require("bn.js"));
exports.BPF_LOADER_UPGRADEABLE_PROGRAM_ID = new web3_js_1.PublicKey("BPFLoaderUpgradeab1e11111111111111111111111");
function programDataAddress(programId) {
    return web3_js_1.PublicKey.findProgramAddressSync([new web3_js_1.PublicKey(programId).toBytes()], exports.BPF_LOADER_UPGRADEABLE_PROGRAM_ID)[0];
}
exports.programDataAddress = programDataAddress;
exports.pubKeyConversion = {
    to: (encoded) => new web3_js_1.PublicKey(encoded),
    from: (decoded) => decoded.toBytes(),
};
//neither anchor nor solana web3 have a built-in way to parse this, because ofc they don't
exports.programDataLayout = [
    { name: "slot", binary: "uint", endianness: "little", size: 8 },
    {
        name: "upgradeAuthority",
        binary: "switch",
        idSize: 1,
        idTag: "isSome",
        layouts: [
            [[0, false], []],
            [
                [1, true],
                [
                    {
                        name: "value",
                        binary: "bytes",
                        size: 32,
                        custom: exports.pubKeyConversion,
                    },
                ],
            ],
        ],
    },
];
exports.U64 = {
    MAX: new bn_js_1.default((2n ** 64n - 1n).toString()),
    to: (amount, unit) => {
        const ret = new bn_js_1.default(Math.round(amount * unit));
        if (ret.isNeg())
            throw new Error("Value negative");
        if (ret.bitLength() > 64)
            throw new Error("Value too large");
        return ret;
    },
    from: (amount, unit) => amount.toNumber() / unit,
};
function derivePda(seeds, programId) {
    const toBytes = (s) => typeof s === "string" ? sdk_connect_1.encoding.bytes.encode(s) : s;
    return web3_js_1.PublicKey.findProgramAddressSync(Array.isArray(seeds) ? seeds.map(toBytes) : [toBytes(seeds)], new web3_js_1.PublicKey(programId))[0];
}
exports.derivePda = derivePda;
const chainToBytes = (chain) => sdk_connect_1.encoding.bignum.toBytes((0, sdk_connect_1.toChainId)(chain), 2);
// TODO: memoize?
const nttAddresses = (programId) => {
    const configAccount = () => derivePda("config", programId);
    const emitterAccount = () => derivePda("emitter", programId);
    const inboxRateLimitAccount = (chain) => derivePda(["inbox_rate_limit", chainToBytes(chain)], programId);
    const inboxItemAccount = (chain, nttMessage) => derivePda(["inbox_item", sdk_definitions_ntt_1.Ntt.messageDigest(chain, nttMessage)], programId);
    const outboxRateLimitAccount = () => derivePda("outbox_rate_limit", programId);
    const tokenAuthority = () => derivePda("token_authority", programId);
    const peerAccount = (chain) => derivePda(["peer", chainToBytes(chain)], programId);
    const transceiverPeerAccount = (chain) => derivePda(["transceiver_peer", chainToBytes(chain)], programId);
    const registeredTransceiver = (transceiver) => derivePda(["registered_transceiver", transceiver.toBytes()], programId);
    const transceiverMessageAccount = (chain, id) => derivePda(["transceiver_message", chainToBytes(chain), id], programId);
    const wormholeMessageAccount = (outboxItem) => derivePda(["message", outboxItem.toBytes()], programId);
    const sessionAuthority = (sender, args) => derivePda([
        "session_authority",
        sender.toBytes(),
        (0, sdk_connect_1.keccak256)(sdk_connect_1.encoding.bytes.concat(sdk_connect_1.encoding.bignum.toBytes(args.amount, 8), chainToBytes(args.recipient.chain), args.recipient.address.toUniversalAddress().toUint8Array(), new Uint8Array([args.shouldQueue ? 1 : 0]))),
    ], programId);
    return {
        configAccount,
        outboxRateLimitAccount,
        inboxRateLimitAccount,
        inboxItemAccount,
        sessionAuthority,
        tokenAuthority,
        emitterAccount,
        wormholeMessageAccount,
        peerAccount,
        transceiverPeerAccount,
        transceiverMessageAccount,
        registeredTransceiver,
    };
};
exports.nttAddresses = nttAddresses;
//# sourceMappingURL=utils.js.map